@@ -1,12 +1,13 @@
 """LSP client for Pyright language server."""
 
-from .client import LspClient, get_lsp_client, close_all_clients
+from .client import LspClient, get_lsp_client, close_all_clients, refresh_lsp_documents
 from .types import Position, Range, Location, TextDocumentIdentifier
 
 __all__ = [
     "LspClient",
     "get_lsp_client",
     "close_all_clients",
+    "refresh_lsp_documents",
     "Position",
     "Range",
     "Location",
