@@ -2,6 +2,9 @@
 
 import atexit
 import json
+import os
+import sys
+import threading
 from typing import Literal, Optional
 
 from mcp.server.fastmcp import FastMCP
@@ -17,6 +20,9 @@ from .rope_client import get_client as get_rope_client
 from .lsp import get_lsp_client, close_all_clients
 from .tools import (
     do_rename,
+    do_move,
+    do_change_signature,
+    get_function_signature,
     get_completions as rope_completions,
     get_definition as rope_definition,
     get_hover as rope_hover,
@@ -75,8 +81,14 @@ def hover(
             client = get_lsp_client(workspace)
             result = client.hover(file, line, column)
             if result:
-                return json.dumps({"contents": result.get("contents", ""), "backend": "pyright"}, indent=2)
-            return json.dumps({"contents": None, "message": "No hover info", "backend": "pyright"}, indent=2)
+                return json.dumps(
+                    {"contents": result.get("contents", ""), "backend": "pyright"},
+                    indent=2,
+                )
+            return json.dumps(
+                {"contents": None, "message": "No hover info", "backend": "pyright"},
+                indent=2,
+            )
         except Exception as e:
             return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
     else:
@@ -114,7 +126,10 @@ def definition(
                 result = locations[0]
                 result["backend"] = "pyright"
                 return json.dumps(result, indent=2)
-            return json.dumps({"file": None, "message": "No definition found", "backend": "pyright"}, indent=2)
+            return json.dumps(
+                {"file": None, "message": "No definition found", "backend": "pyright"},
+                indent=2,
+            )
         except Exception as e:
             return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
     else:
@@ -148,7 +163,9 @@ def references(
             workspace = _find_workspace(file)
             client = get_lsp_client(workspace)
             refs = client.references(file, line, column)
-            return json.dumps({"references": refs, "count": len(refs), "backend": "pyright"}, indent=2)
+            return json.dumps(
+                {"references": refs, "count": len(refs), "backend": "pyright"}, indent=2
+            )
         except Exception as e:
             return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
     else:
@@ -182,7 +199,10 @@ def completions(
             workspace = _find_workspace(file)
             client = get_lsp_client(workspace)
             items = client.completions(file, line, column)
-            return json.dumps({"completions": items, "count": len(items), "backend": "pyright"}, indent=2)
+            return json.dumps(
+                {"completions": items, "count": len(items), "backend": "pyright"},
+                indent=2,
+            )
         except Exception as e:
             return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
     else:
@@ -218,7 +238,15 @@ def symbols(
             if query:
                 query_lower = query.lower()
                 syms = [s for s in syms if query_lower in s["name"].lower()]
-            return json.dumps({"symbols": syms, "count": len(syms), "file": file, "backend": "pyright"}, indent=2)
+            return json.dumps(
+                {
+                    "symbols": syms,
+                    "count": len(syms),
+                    "file": file,
+                    "backend": "pyright",
+                },
+                indent=2,
+            )
         except Exception as e:
             return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
     else:
@@ -248,6 +276,116 @@ def rename(file: str, line: int, column: int, new_name: str) -> str:
     return json.dumps(result, indent=2)
 
 
+@mcp.tool()
+def move(
+    file: str,
+    line: int,
+    column: int,
+    destination: str,
+    preview: bool = False,
+) -> str:
+    """Move a function or class to another module.
+
+    This will modify files on disk to move the symbol and update all imports.
+    Uses Rope backend for refactoring.
+
+    Args:
+        file: Absolute path to the Python file containing the symbol
+        line: 1-based line number of the symbol to move
+        column: 1-based column number of the symbol
+        destination: Destination module path (e.g., "mypackage.utils" or "utils.py")
+        preview: If True, only show what would change without applying
+
+    Returns:
+        JSON string with changes made or error message
+    """
+    result = do_move(file, line, column, destination, resources_only=preview)
+    result["backend"] = "rope"
+    return json.dumps(result, indent=2)
+
+
+@mcp.tool()
+def change_signature(
+    file: str,
+    line: int,
+    column: int,
+    new_params: Optional[list[str]] = None,
+    add_param: Optional[str] = None,
+    add_param_default: Optional[str] = None,
+    add_param_index: Optional[int] = None,
+    remove_param: Optional[str] = None,
+    preview: bool = False,
+) -> str:
+    """Change the signature of a function.
+
+    This will modify files on disk to update the function and all call sites.
+    Uses Rope backend for refactoring.
+
+    Args:
+        file: Absolute path to the Python file
+        line: 1-based line number of the function
+        column: 1-based column number of the function
+        new_params: New parameter order, e.g. ["self", "b", "a"] to reorder
+        add_param: Name of parameter to add
+        add_param_default: Default value for added parameter
+        add_param_index: Index where to insert new param (None = append)
+        remove_param: Name of parameter to remove
+        preview: If True, only show what would change without applying
+
+    Returns:
+        JSON string with changes made or error message
+
+    Examples:
+        # Reorder: def foo(a, b) -> def foo(b, a)
+        change_signature(file, line, col, new_params=["self", "b", "a"])
+
+        # Add param: def foo(a) -> def foo(a, b=None)
+        change_signature(file, line, col, add_param="b", add_param_default="None")
+
+        # Remove param: def foo(a, b) -> def foo(a)
+        change_signature(file, line, col, remove_param="b")
+    """
+    # Build add_param dict if specified
+    add_param_dict = None
+    if add_param:
+        add_param_dict = {
+            "name": add_param,
+            "default": add_param_default,
+            "index": add_param_index,
+        }
+
+    result = do_change_signature(
+        file,
+        line,
+        column,
+        new_params=new_params,
+        add_param=add_param_dict,
+        remove_param=remove_param,
+        resources_only=preview,
+    )
+    result["backend"] = "rope"
+    return json.dumps(result, indent=2)
+
+
+@mcp.tool()
+def function_signature(file: str, line: int, column: int) -> str:
+    """Get the current signature of a function.
+
+    Useful for inspecting function parameters before changing the signature.
+
+    Args:
+        file: Absolute path to the Python file
+        line: 1-based line number of the function
+        column: 1-based column number of the function
+
+    Returns:
+        JSON string with function signature info
+    """
+    result = get_function_signature(file, line, column)
+    result["backend"] = "rope"
+    return json.dumps(result, indent=2)
+
+
 @mcp.tool()
 def diagnostics(path: str) -> str:
     """Get type errors and warnings for a Python file or directory.
@@ -286,7 +424,9 @@ def signature_help(file: str, line: int, column: int) -> str:
         if result:
             result["backend"] = "pyright"
             return json.dumps(result, indent=2)
-        return json.dumps({"message": "No signature help available", "backend": "pyright"}, indent=2)
+        return json.dumps(
+            {"message": "No signature help available", "backend": "pyright"}, indent=2
+        )
     except Exception as e:
         return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
 
@@ -309,7 +449,9 @@ def update_document(file: str, content: str) -> str:
         workspace = _find_workspace(file)
         client = get_lsp_client(workspace)
         client.update_document(file, content)
-        return json.dumps({"success": True, "file": file, "backend": "pyright"}, indent=2)
+        return json.dumps(
+            {"success": True, "file": file, "backend": "pyright"}, indent=2
+        )
     except Exception as e:
         return json.dumps({"error": str(e), "backend": "pyright"}, indent=2)
 
@@ -365,30 +507,42 @@ def set_backend(
     try:
         backend_enum = Backend(backend.lower())
     except ValueError:
-        return json.dumps({
-            "error": f"Invalid backend: {backend}. Must be 'rope' or 'pyright'.",
-        }, indent=2)
+        return json.dumps(
+            {
+                "error": f"Invalid backend: {backend}. Must be 'rope' or 'pyright'.",
+            },
+            indent=2,
+        )
 
     if tool:
         if tool not in SHARED_TOOLS:
-            return json.dumps({
-                "error": f"Invalid tool: {tool}. Must be one of: {', '.join(SHARED_TOOLS)}",
-            }, indent=2)
+            return json.dumps(
+                {
+                    "error": f"Invalid tool: {tool}. Must be one of: {', '.join(SHARED_TOOLS)}",
+                },
+                indent=2,
+            )
         config.set_backend(backend_enum, tool)
-        return json.dumps({
-            "success": True,
-            "message": f"Backend for '{tool}' set to '{backend}'",
-            "tool": tool,
-            "backend": backend,
-        }, indent=2)
+        return json.dumps(
+            {
+                "success": True,
+                "message": f"Backend for '{tool}' set to '{backend}'",
+                "tool": tool,
+                "backend": backend,
+            },
+            indent=2,
+        )
     else:
         config.set_all_backends(backend_enum)
-        return json.dumps({
-            "success": True,
-            "message": f"Default backend set to '{backend}' for all shared tools",
-            "backend": backend,
-            "affected_tools": list(SHARED_TOOLS),
-        }, indent=2)
+        return json.dumps(
+            {
+                "success": True,
+                "message": f"Default backend set to '{backend}' for all shared tools",
+                "backend": backend,
+                "affected_tools": list(SHARED_TOOLS),
+            },
+            indent=2,
+        )
 
 
 @mcp.tool()
@@ -442,6 +596,8 @@ def status() -> str:
                 "description": "Fast, Python-native code analysis",
                 "active_projects": active_projects,
                 "project_python_paths": project_python_paths,
+                "caching_enabled": rope_status.get("caching_enabled", False),
+                "cache_folder": rope_status.get("cache_folder"),
             },
             "pyright": {
                 "available": True,
@@ -470,9 +626,215 @@ def status() -> str:
     return json.dumps(status_info, indent=2)
 
 
+@mcp.tool()
+def reload_modules() -> str:
+    """Reload all tool modules (development only).
+
+    This reloads the Python modules so code changes take effect without
+    restarting the server. Use this during development/debugging.
+
+    Returns:
+        JSON string with reload status
+    """
+    import importlib
+    from . import tools
+    from .tools import (
+        hover,
+        definition,
+        references,
+        completions,
+        symbols,
+        rename,
+        move,
+        change_signature,
+        diagnostics,
+        search,
+    )
+    from . import rope_client
+    from . import config
+    from .lsp import client as lsp_client
+
+    reloaded = []
+    errors = []
+
+    # Reload in dependency order
+    modules_to_reload = [
+        ("config", config),
+        ("rope_client", rope_client),
+        ("lsp.client", lsp_client),
+        ("tools.hover", hover),
+        ("tools.definition", definition),
+        ("tools.references", references),
+        ("tools.completions", completions),
+        ("tools.symbols", symbols),
+        ("tools.rename", rename),
+        ("tools.move", move),
+        ("tools.change_signature", change_signature),
+        ("tools.diagnostics", diagnostics),
+        ("tools.search", search),
+        ("tools", tools),
+    ]
+
+    for name, module in modules_to_reload:
+        try:
+            importlib.reload(module)
+            reloaded.append(name)
+        except Exception as e:
+            errors.append({"module": name, "error": str(e)})
+
+    # Re-import the functions we use
+    global do_rename, do_move, do_change_signature, get_function_signature
+    global rope_completions, rope_definition, rope_hover, rope_references, rope_symbols
+    global get_diagnostics, get_search
+
+    from .tools import (
+        do_rename,
+        do_move,
+        do_change_signature,
+        get_function_signature,
+        get_completions as rope_completions,
+        get_definition as rope_definition,
+        get_hover as rope_hover,
+        get_references as rope_references,
+        get_symbols as rope_symbols,
+        get_diagnostics,
+        get_search,
+    )
+
+    return json.dumps(
+        {
+            "success": len(errors) == 0,
+            "reloaded": reloaded,
+            "errors": errors if errors else None,
+        },
+        indent=2,
+    )
+
+
+def _do_reload():
+    """Perform module reload."""
+    import importlib
+    from . import tools
+    from .tools import (
+        hover,
+        definition,
+        references,
+        completions,
+        symbols,
+        rename,
+        move,
+        change_signature,
+        diagnostics,
+        search,
+    )
+    from . import rope_client
+    from . import config
+    from .lsp import client as lsp_client
+
+    # Reload in dependency order
+    modules = [
+        config,
+        rope_client,
+        lsp_client,
+        hover,
+        definition,
+        references,
+        completions,
+        symbols,
+        rename,
+        move,
+        change_signature,
+        diagnostics,
+        search,
+        tools,
+    ]
+
+    for module in modules:
+        try:
+            importlib.reload(module)
+        except Exception as e:
+            print(f"Error reloading {module.__name__}: {e}", file=sys.stderr)
+
+    # Re-import the functions
+    global do_rename, do_move, do_change_signature, get_function_signature
+    global rope_completions, rope_definition, rope_hover, rope_references, rope_symbols
+    global get_diagnostics, get_search
+
+    from .tools import (
+        do_rename,
+        do_move,
+        do_change_signature,
+        get_function_signature,
+        get_completions as rope_completions,
+        get_definition as rope_definition,
+        get_hover as rope_hover,
+        get_references as rope_references,
+        get_symbols as rope_symbols,
+        get_diagnostics,
+        get_search,
+    )
+
+    print("Modules reloaded", file=sys.stderr)
+
+
+def _start_file_watcher(src_path: str):
+    """Start watching for file changes and reload modules."""
+    try:
+        from watchdog.observers import Observer
+        from watchdog.events import FileSystemEventHandler, FileModifiedEvent
+    except ImportError:
+        print(
+            "watchdog not installed. Run: uv pip install watchdog",
+            file=sys.stderr,
+        )
+        return None
+
+    class ReloadHandler(FileSystemEventHandler):
+        def __init__(self):
+            self._debounce_timer = None
+            self._lock = threading.Lock()
+
+        def _schedule_reload(self):
+            with self._lock:
+                if self._debounce_timer:
+                    self._debounce_timer.cancel()
+                self._debounce_timer = threading.Timer(0.5, self._do_reload)
+                self._debounce_timer.start()
+
+        def _do_reload(self):
+            try:
+                _do_reload()
+            except Exception as e:
+                print(f"Reload error: {e}", file=sys.stderr)
+
+        def on_modified(self, event):
+            if isinstance(event, FileModifiedEvent) and event.src_path.endswith(".py"):
+                print(f"File changed: {event.src_path}", file=sys.stderr)
+                self._schedule_reload()
+
+    observer = Observer()
+    observer.schedule(ReloadHandler(), src_path, recursive=True)
+    observer.start()
+    print(f"Watching for changes in: {src_path}", file=sys.stderr)
+    return observer
+
+
 def main():
     """Run the MCP server."""
-    mcp.run()
+    reload_mode = "--reload" in sys.argv or os.environ.get("MCP_RELOAD") == "1"
+
+    observer = None
+    if reload_mode:
+        # Get the source path
+        src_path = os.path.dirname(os.path.abspath(__file__))
+        observer = _start_file_watcher(src_path)
+
+    try:
+        mcp.run()
+    finally:
+        if observer:
+            observer.stop()
+            observer.join()
 
 
 if __name__ == "__main__":
