@@ -6,6 +6,8 @@ from .references import get_references
 from .completions import get_completions
 from .symbols import get_symbols
 from .rename import do_rename
+from .move import do_move
+from .change_signature import do_change_signature, get_function_signature
 from .diagnostics import get_diagnostics
 from .search import get_search
 
@@ -16,6 +18,9 @@ __all__ = [
     "get_completions",
     "get_symbols",
     "do_rename",
+    "do_move",
+    "do_change_signature",
+    "get_function_signature",
     "get_diagnostics",
     "get_search",
 ]
